from afeng_tools.fastapi_tool.common.po_service.base_po_service import PoService


class TmpSortPoService(PoService):
    """
    使用示例：tmp_sort_po_service = TmpSortPoService(app_info.db_code, TmpSortInfoPo)
    使用步骤如下：
        1、使用查询参数生成 unique_code
        2、使用 unique_code 查询是否已经有缓存排序数据
            tmp_po_list = tmp_sort_po_service.query_more(TmpSortInfoPo.unique_code == unique_code,
                                                 sort_column='sort_index',
                                                 sort_type=SortTypeEnum.asc)
            - 如果存在，则使用缓存的排序数据
                match_code_list = [tmp.sort_value for tmp in tmp_po_list]
            - 如果不存在，则查询排序后的数据，然后入库
                match_code_list = 根据自己的需要进行查询
                tmp_sort_po_service.add_in_list(unique_code, match_code_list)
        3、将已有的 db_query 和 缓存的排序数据表进行拼接，根据排序表中的索引进行排序
            db_query = db_query.join(TmpSortInfoPo, DocInfoPo.code == TmpSortInfoPo.sort_value,
                        isouter=True).filter(TmpSortInfoPo.unique_code == unique_code)
            db_query = db_query.order_by(TmpSortInfoPo.sort_index.asc())
    """
    _table_name_ = "tmp_sort_info"

    def delete_by_unique_code(self, unique_code: str):
        """通过唯一码删除"""
        return self.delete(self.model_type.unique_code == unique_code)

    def add_in_list(self, unique_code: str, in_data_list: list):
        """添加in列表"""
        tmp_po_list = []
        for index, tmp_in_data in enumerate(in_data_list):
            tmp_po_list.append(self.model_type(
                type_code='default',
                unique_code=unique_code,
                sort_value=str(tmp_in_data),
                sort_index=(index + 1)
            ))
        return self.add_batch(tmp_po_list)





from typing import Optional

from pydantic import BaseModel

from afeng_tools.quark_tool.core.quark_items import ShareToken, PageMetadata, ShareData


class QuarkResponse(BaseModel):
    # ok：200
    status: Optional[int] = None
    # 0
    code: Optional[int] = None
    # ok
    message: Optional[str] = None
    timestamp: Optional[int] = None


class ShareTokenResponse(QuarkResponse):
    data: Optional[ShareToken] = None


class ShareDataResponse(QuarkResponse):
    metadata: Optional[PageMetadata] = None
    data: Optional[ShareData] = None



from typing import Optional, Any, List

from pydantic import BaseModel, Field


class ShareUser(BaseModel):
    # 如：SUPER_VIP
    member_type: Optional[str] = None
    avatar_url: Optional[str] = None
    nick_name: Optional[str] = None


class ShareToken(BaseModel):
    # 如：英语兔英语词汇音标发音学习
    title: Optional[str] = None
    subscribed: Optional[bool] = None
    stoken: Optional[str] = None
    share_type: Optional[int] = None
    author: Optional[ShareUser] = None
    expired_type: Optional[int] = None
    expired_at: Optional[int] = None
    pwd_id: Optional[str] = None


class PageMetadata(BaseModel):
    check_fid_token: Optional[int] = None
    count: Optional[int] = Field(alias='_count',default=None)
    page: Optional[int] = Field(alias='_page',default=None)
    size: Optional[int] = Field(alias='_size',default=None)
    total: Optional[int] = Field(alias='_total',default=None)


class ShareBaseFile(BaseModel):
    fid: Optional[str] = None
    # 0、文件夹， 1、图片、视频
    file_type: Optional[int] = None
    # "image/png"、 "video/mp4"
    format_type: Optional[str] = None
    # 如： "image"、  "video"
    obj_category: Optional[int] = None
    name_space: Optional[int] = None
    series_dir: Optional[bool] = None
    upload_camera_root_dir: Optional[bool] = None
    fps: Optional[float] = None
    like: Optional[int] = None
    risk_type: Optional[int] = None
    file_name_hl_start: Optional[int] = None
    file_name_hl_end: Optional[int] = None
    duration: Optional[int] = None
    ban: Optional[bool] = None
    cur_version_or_default: Optional[int] = None
    offline_source: Optional[bool] = None
    backup_source: Optional[bool] = None
    save_as_source: Optional[bool] = None
    owner_drive_type_or_default: Optional[int] = None
    dir: Optional[bool] = None
    file: Optional[bool] = None
    extra: Optional[dict[str, Any]] = Field(alias='_extra', default=None, extra=False)


class ShareFileStruct(BaseModel):
    # 示例：saveas
    fir_source: Optional[str] = None
    # 示例：share_save
    sec_source: Optional[str] = None
    # 示例：share_save
    thi_source: Optional[str] = None
    # 示例：android
    platform_source: Optional[str] = None


class ShareFile(ShareBaseFile):
    file_name: Optional[str] = None
    pdir_fid: Optional[str] = None
    category: Optional[int] = None
    size: Optional[int] = None
    status: Optional[int] = None
    tags: Optional[str] = None
    owner_ucid: Optional[str] = None
    share_fid_token: Optional[str] = None
    created_at: Optional[int] = None
    updated_at: Optional[int] = None
    last_update_at: Optional[int] = None
    l_created_at: Optional[int] = None
    l_updated_at: Optional[int] = None
    l_shot_at: Optional[int] = None
    # 图片、视频有缩略图
    big_thumbnail: Optional[str] = None
    thumbnail: Optional[str] = None
    # 视频的属性，示例：1080
    video_height: Optional[int] = None
    # 示例：1920
    video_width: Optional[int] = None
    # 示例："super"
    video_max_resolution: Optional[str] = None
    # 示例：-1
    video_rotate: Optional[int] = None
    operated_at: Optional[int] = None
    include_items: Optional[int] = None
    # 示例：ucpro-android:saveas
    source: Optional[str] = None
    # 示例:UCPRO-ANDROID:SAVE_SHARE
    file_source: Optional[str] = None
    # 示例：save_share
    source_display: Optional[str] = None
    name_space: Optional[int] = None
    series_dir: Optional[bool] = None
    upload_camera_root_dir: Optional[bool] = None
    backup_sign: Optional[int] = None
    file_struct: Optional[ShareFileStruct] = None
    extra: Optional[str] = None
    event_extra: Optional[dict[str, Any]] = None
    raw_name_space: Optional[int] = None


class ShareItem(BaseModel):
    title: Optional[str] = None
    path_info: Optional[str] = None
    size: Optional[int] = None
    share_id: Optional[str] = None
    pwd_id: Optional[str] = None
    share_url: Optional[str] = None
    file_num: Optional[int] = None
    first_fid: Optional[str] = None
    first_file: Optional[ShareBaseFile] = None
    share_type: Optional[int] = None
    url_type: Optional[int] = None
    expired_type: Optional[int] = None
    created_at: Optional[int] = None
    updated_at: Optional[int] = None
    expired_at: Optional[int] = None
    expired_left: Optional[int] = None
    audit_status: Optional[int] = None
    status: Optional[int] = None
    click_pv: Optional[int] = None
    save_pv: Optional[int] = None
    download_pv: Optional[int] = None
    partial_violation: Optional[bool] = None
    first_layer_file_categories: Optional[List[int]] = None
    is_owner: Optional[int] = None
    download_pvlimited: Optional[bool] = None
    pass


class ShareData(BaseModel):
    is_owner: Optional[int] = None
    share: Optional[ShareItem] = None
    list: Optional[List[ShareFile]] = None



"""
夸克分享工具
"""
import json
import time

import urllib.parse

from afeng_tools.quark_tool.core import quark_http_tools
from afeng_tools.quark_tool.core.quark_items import ShareToken, ShareData, PageMetadata
from afeng_tools.quark_tool.core.quark_response_items import ShareTokenResponse, ShareDataResponse


def get_share_token(pwd_id: str, pwd: str = '') -> ShareToken:
    resp_json = quark_http_tools.post(
        url=f'https://drive-pc.quark.cn/1/clouddrive/share/sharepage/token?pr=ucpro&fr=pc&uc_param_str=&__dt=1653&__t={int(time.time())}138',
        json_params={
            'pwd_id': pwd_id,
            'passcode': pwd
        })
    if resp_json:
        response = ShareTokenResponse(**resp_json)
        if response.status == 200:
            share_token = response.data
            share_token.pwd_id = pwd_id
            return share_token


def get_share_info(share_token: ShareToken, pdir_fid: str = '0', page_num: int = 1, page_size: int = 50) -> tuple[
    PageMetadata, ShareData]:
    resp_json = quark_http_tools.get(
        url=f'https://drive-pc.quark.cn/1/clouddrive/share/sharepage/detail?pr=ucpro&fr=pc&uc_param_str='
            f'&pwd_id={share_token.pwd_id}'
            f'&stoken={urllib.parse.quote(share_token.stoken)}'
            f'&pdir_fid={pdir_fid}&force=0'
            f'&_page={page_num}&_size={page_size}&_fetch_banner=1&_fetch_share=1&_fetch_total=1'
            f'&_sort=file_type:asc,updated_at:desc&__dt=2129&__t={int(time.time())}374')
    if resp_json:
        response = ShareDataResponse(**resp_json)
        if response.status == 200:
            return response.metadata, response.data


if __name__ == '__main__':
    # tmp_share_token = get_share_token(pwd_id='de8e831c4e4f')
    # print(json.dumps(tmp_share_token.model_dump(mode='json'), ensure_ascii=False, indent=4))
    # print(int(time.time()))
    tmp_share_token = ShareToken(pwd_id='de8e831c4e4f', stoken="AmDTDLfPcWmN+xqB+vWvfVB82vJ8uwZz48c8V6WTdLs=")
    page_data, share_data = get_share_info(tmp_share_token)
    print()
    pass



from typing import Any

import requests

from afeng_tools.quark_tool.core.quark_response_items import QuarkResponse


def create_default_headers() -> dict[str, str]:
    """创建默认header"""
    return {
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/107.0.0.0 Safari/537.36 Edg/107.0.1418.24',
        'Origin': 'https://pan.quark.cn',
        'Referer': 'https://pan.quark.cn',
    }


def post(url, json_params: dict[str, Any] = None) -> dict[str, Any]:
    response = requests.post(url, headers=create_default_headers(), json=json_params)
    if response.status_code == 200:
        response.encoding = 'utf-8'
        return response.json()


def get(url) -> dict[str, Any]:
    response = requests.get(url, headers=create_default_headers())
    if response.status_code == 200:
        response.encoding = 'utf-8'
        return response.json()



"""
模块页面主模块渲染服务
"""
from typing import Callable

from afeng_tools.fastapi_tool.common.enum import ReloadFreqEnum
from afeng_tools.fastapi_tool.template.item import TemplateHtmlHeadData, TemplateTopBarData, TemplateLogoSearchData, \
    TemplateIndexPageHeaderData, TemplatePageSearchHeaderData, TemplatePageHeaderData, TemplateFixNavData
from afeng_tools.fastapi_tool.template.template_render_service.template_render_tools import render_template_file
from afeng_tools.template_tool.template_decorator_tools import template_area


@template_area(file='views/head/page_head.html')
def get_template_head(group_code: str, context_data_func: Callable[[], TemplateHtmlHeadData],
                      app_code: str = 'common', reload_freq: ReloadFreqEnum = ReloadFreqEnum.always,
                      is_mobile: bool = True,
                      template_file: str = None) -> str:
    def context_data_func_wrapper():
        data = context_data_func()
        context_data = data.model_dump()
        context_data['head_title'] = data.title
        context_data['head_description'] = data.description
        context_data['head_keywords'] = ','.join([tmp for tmp in data.keyword_list if tmp])
        context_data['head_author'] = data.author
        context_data['head_favicon'] = data.favicon
        context_data['head_origin'] = data.origin
        return context_data

    return render_template_file(app_code, template_file,
                                group_code=group_code, context_data_func=context_data_func_wrapper,
                                reload_freq=reload_freq, is_mobile=is_mobile)


@template_area(file='module_html/header/top_bar.html')
def get_template_top_bar(group_code: str, context_data_func: Callable[[], TemplateTopBarData],
                         app_code: str = 'common', reload_freq: ReloadFreqEnum = ReloadFreqEnum.always,
                         is_mobile: bool = True,
                         template_file: str = None) -> str:
    """
    页面顶部topbar模板
    :param group_code: 分组编码
    :param context_data_func: 上下文数据生成函数
    :param app_code
    :param reload_freq:
    :param is_mobile:
    :param template_file:
    :return:
    """

    def context_data_func_wrapper():
        data = context_data_func()
        return data.model_dump()

    return render_template_file(app_code, template_file,
                                group_code=group_code, context_data_func=context_data_func_wrapper,
                                reload_freq=reload_freq, is_mobile=is_mobile)


@template_area(file='module_html/header/logo_search.html')
def get_template_logo_search(group_code: str, context_data_func: Callable[[], TemplateLogoSearchData],
                             app_code: str = 'common', reload_freq: ReloadFreqEnum = ReloadFreqEnum.always,
                             is_mobile: bool = True, template_file: str = None) -> str:
    """
    页面顶部logo_search模板
    :param is_mobile:
    :param app_code:
    :param group_code: 分组编码
    :param context_data_func: 上下文数据生成函数
    :param reload_freq:
    :param template_file:
    :return:
    """

    def context_data_func_wrapper():
        data = context_data_func()
        return data.model_dump()

    return render_template_file(app_code, template_file,
                                group_code=group_code, context_data_func=context_data_func_wrapper,
                                reload_freq=reload_freq, is_mobile=is_mobile)


@template_area(file='module_html/header/fix_nav.html')
def get_template_fix_nav(group_code: str, context_data_func: Callable[[], TemplateFixNavData],
                         app_code: str = 'common', reload_freq: ReloadFreqEnum = ReloadFreqEnum.always,
                         is_mobile: bool = True, template_file: str = None) -> str:
    """
    页面顶部logo_search模板
    :param is_mobile:
    :param app_code:
    :param group_code: 分组编码
    :param context_data_func: 上下文数据生成函数
    :param reload_freq:
    :param template_file:
    :return:
    """

    def context_data_func_wrapper():
        data = context_data_func()
        return data.model_dump()

    return render_template_file(app_code, template_file,
                                group_code=group_code, context_data_func=context_data_func_wrapper,
                                reload_freq=reload_freq, is_mobile=is_mobile)


@template_area(file='module_html/page/page_header.html')
def get_template_page_header(group_code: str, context_data_func: Callable[[], TemplatePageHeaderData],
                             app_code: str = 'common', reload_freq: ReloadFreqEnum = ReloadFreqEnum.always,
                             is_mobile: bool = True, template_file: str = None) -> str:
    """
    页面顶部模板
    :param is_mobile:
    :param app_code:
    :param group_code: 分组编码
    :param context_data_func: 上下文数据生成函数
    :param reload_freq:
    :param template_file:
    :return:
    """

    def context_data_func_wrapper():
        data = context_data_func()
        context_data = data.model_dump()
        if 'is_mobile' not in context_data:
            context_data['is_mobile'] = data.is_mobile
        if 'template_top_bar' not in context_data and data.topbar_data:
            context_data['template_top_bar'] = get_template_top_bar(app_code=app_code, group_code=group_code,
                                                                    context_data_func=lambda: data.topbar_data,
                                                                    reload_freq=reload_freq, is_mobile=is_mobile)
        if 'template_logo_search' not in context_data and data.logo_search_data:
            context_data['template_logo_search'] = get_template_logo_search(app_code=app_code, group_code=group_code,
                                                                            context_data_func=lambda: data.logo_search_data,
                                                                            reload_freq=reload_freq,
                                                                            is_mobile=is_mobile)
        if 'template_fix_nav' not in context_data and data.fix_nav_data:
            context_data['template_fix_nav'] = get_template_fix_nav(app_code=app_code, group_code=group_code,
                                                                    context_data_func=lambda: data.fix_nav_data,
                                                                    reload_freq=reload_freq, is_mobile=is_mobile)
        return context_data

    return render_template_file(app_code, template_file,
                                group_code=group_code, context_data_func=context_data_func_wrapper,
                                reload_freq=reload_freq, is_mobile=is_mobile)


@template_area(file='module_html/page/page_index_header.html')
def get_template_index_page_header(group_code: str, context_data_func: Callable[[], TemplateIndexPageHeaderData],
                                   app_code: str = 'common', reload_freq: ReloadFreqEnum = ReloadFreqEnum.always,
                                   is_mobile: bool = True,
                                   template_file: str = None) -> str:
    """
    首页页面顶部模板
    :param app_code:
    :param is_mobile:
    :param group_code: 分组编码
    :param context_data_func: 上下文数据生成函数
    :param reload_freq:
    :param template_file:
    :return:
    """

    def context_data_func_wrapper():
        data = context_data_func()
        return data.model_dump()

    return render_template_file(app_code, template_file,
                                group_code=group_code, context_data_func=context_data_func_wrapper,
                                reload_freq=reload_freq, is_mobile=is_mobile)


@template_area(file='module_html/page/page_search_header.html')
def get_template_page_search_header(group_code: str, context_data_func: Callable[[], TemplatePageSearchHeaderData],
                                    app_code: str = 'common', reload_freq: ReloadFreqEnum = ReloadFreqEnum.always,
                                    is_mobile: bool = True,
                                    template_file: str = None) -> str:
    """
    页面搜索顶部模板
    :param is_mobile:
    :param app_code:
    :param group_code: 分组编码
    :param context_data_func: 上下文数据生成函数
    :param reload_freq:
    :param template_file:
    :return:
    """

    def context_data_func_wrapper():
        data = context_data_func()
        return data.model_dump()

    return render_template_file(app_code, template_file,
                                group_code=group_code, context_data_func=context_data_func_wrapper,
                                reload_freq=reload_freq, is_mobile=is_mobile)



"""
模块渲染工具
"""
import os
from typing import Callable

from afeng_tools.application_tool import settings_tools
from afeng_tools.application_tool.settings_enum import SettingsKeyEnum
from afeng_tools.fastapi_tool.common.enum import ReloadFreqEnum
from afeng_tools.fastapi_tool.template import fastapi_template_tools
from afeng_tools.http_tool.http_url_tool import join_local_url
from afeng_tools.jinja2_tool import jinja2_tools

template_directory = settings_tools.get_config(SettingsKeyEnum.server_template_path)
template_env = jinja2_tools.create_template_env(directory_list=[template_directory])
is_debug = settings_tools.get_config('app.is_debug')


def render(do_render_func: Callable, **kwargs) -> str:
    """渲染"""
    return do_render_func(**kwargs)


def render_template_file(app_code: str, template_file: str, group_code: str,
                         context_data_func: Callable,
                         reload_freq: ReloadFreqEnum = ReloadFreqEnum.weekly,
                         is_mobile: bool = True,
                         **kwargs) -> str:
    """渲染模板文件"""

    template_html_file = join_local_url([app_code, template_file])
    tmp_save_html_file = join_local_url(['tmp', app_code, group_code, 'mobile' if is_mobile else 'pc', template_file])
    save_template_file = str(os.path.join(template_directory, tmp_save_html_file.removeprefix('/')))
    if fastapi_template_tools.is_need_reload(save_template_file, reload_freq, is_debug=is_debug):
        if kwargs:
            context_data = context_data_func(**kwargs)
        else:
            context_data = context_data_func()
        template_content = jinja2_tools.format_template(template_html_file, context=context_data, env=template_env)
        os.makedirs(os.path.dirname(save_template_file), exist_ok=True)
        with open(save_template_file, 'w', encoding='utf-8') as f:
            f.write('{% raw %}' + template_content + '{% endraw %}')
    return tmp_save_html_file



from typing import Optional

from pydantic import BaseModel


class ServiceItem(BaseModel):
    """服务项"""
    # 服务标题
    title: Optional[str] = None


class LinkServiceItem(ServiceItem):
    """链接服务项"""
    # 链接url
    href: Optional[str] = None


class WeixinMpServiceItem(ServiceItem):
    """微信小程序服务项"""
    # 小程序 Appid
    app_id: Optional[str] = None
    # 页面路径
    path: Optional[str] = '/pages/index/index'

